--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.


package body Scalada.Guitar_Fretboards is

   ------------
   -- Create --
   ------------
   function Create
     (Number_Of_Strings : in String_Number := DEFAULT_NUMBER_OF_STRINGS;
      Frets_Number : in Fret_Number := DEFAULT_NUMBER_OF_FRETS;
      Used_Tuning : in Tuning := Tuning'(1 => E,
                                               2 => A,
                                               3 => D,
                                               4 => G,
                                               5 => B,
                                               6 => E
                                        ))
     return Fretboard
   is
      Returned_Fretboard : Fretboard(1..Number_Of_Strings, 0..Frets_Number);
      String_Index : String_Number := 1;
   begin
      -- fill up the fretboard
      for Tuning_Index in Used_Tuning'Range loop
         declare
            Current_Note_index : Note_Index := Note'Pos(Used_Tuning(Tuning_Index));
         begin
            for Fret_Index in Returned_Fretboard'Range(2) loop
               Returned_Fretboard(String_Index, Fret_Index) := Fretboard_Note'(Note'Val(Current_Note_Index), False);
               Current_Note_Index := Current_Note_Index + 1;
            end loop;

            -- to avoid constraint error, check that we can increment
            if String_Index < String_Number'Last then
               String_Index := String_Index + 1;
            end if;
         end;
      end loop;

      return Returned_Fretboard;
   end Create;

   procedure Activate_Notes(On_Fretboard : in out Fretboard; With_Notes : in Set_Of_Notes) is
   begin
      -- brute force method
      for Note_Index in With_Notes'Range loop
         for String_Index in On_Fretboard'Range(1) loop
            for Fret_Index in On_Fretboard'Range(2) loop
               if On_Fretboard(String_Index, Fret_Index).Fret_Note = With_Notes(Note_Index) then
                  On_Fretboard(String_Index, Fret_Index).Fretted := True;
               end if;
            end loop;
         end loop;
      end loop;
   end;

end Scalada.Guitar_Fretboards;
