--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Scalada.Notes; use Scalada.Notes;
with Scalada.NamedStructures; use Scalada.NamedStructures;

package Scalada.Guitar_Fretboards is
   -- Define a type for string indexing
   -- supports 6 strings or 7 strings (or less !!) guitars
   type String_Number is new Positive range 1..7;

   -- Define a type for fret indexing
   -- supports up to 24 frets
   type Fret_Number is new Natural range 0..24;

   type Tuning is array(String_Number range <>) of Note;

   -- this record defines the note on the fretboard
   -- ad its state (fretted or not)
   type Fretboard_Note is record
      Fret_Note : Note;
      Fretted : Boolean;
   end record;

   DEFAULT_NUMBER_OF_STRINGS : constant := 6;
   DEFAULT_NUMBER_OF_FRETS : constant := 24;

   -- this type defines the whole fretboard
   type Fretboard is array(String_Number range <>, Fret_Number range <>) of Fretboard_Note;

   function Create(Number_Of_Strings : in String_Number := DEFAULT_NUMBER_OF_STRINGS;
                   Frets_Number : in Fret_Number := DEFAULT_NUMBER_OF_FRETS;
                   Used_Tuning : in Tuning := Tuning'(1 => E,
                                               2 => A,
                                               3 => D,
                                               4 => G,
                                               5 => B,
                                               6 => E
                                              )) return Fretboard;

   procedure Activate_Notes(On_Fretboard : in out Fretboard; With_Notes : in Set_Of_Notes);

end Scalada.Guitar_Fretboards;
