--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Scalada.Notes;

package Scalada.Intervals is
   pragma Pure(Intervals);

   --Interval in semi-tone between to consecutive plain note
   type Notes_Interval is range 0..(Notes.Note'Pos(Notes.Note'Last) - Notes.Note'Pos(Notes.Note'First));

   -- Intervals as distance from the tonic
   type Quality_Interval is (Tone, Minor_Second, Second, Minor_Third, Major_Third, Perfect_Fourth, Tritone, Perfect_Fifth, Minor_Sixth, Major_Sixth, Minor_Seventh, Major_Seventh);

   -- This function returns the interval from the tonic as number of semi-tones
   function Interval_Value(Of_The_quality : Quality_Interval) return Notes_Interval;

   -- This function returns the interval in semi-tones as quality
   function Quality_Of(Interval : Notes_Interval) return Quality_Interval;

   -- Returns the interval between two notes as a name
   function Quality_Of(Note :Notes.Note; To_Tonic :Notes.Note) return Quality_Interval;

   -- returns the note according to the distance
   function Get_Note(Quality : Quality_Interval; Of_Tonic : Notes.Note) return Notes.Note;

end Scalada.Intervals;
