--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

package body Scalada.NamedStructures is
   -- Utility functions
   function Convert_Intervals(Intervals : Quality_Interval_Array) return Interval_Array is
      Semi_Tones : Interval_Array(Intervals'Range);
   begin
      for Index in Intervals'Range loop
         if (Index + 1 > Intervals'Last) then
            -- Deal with the wrap around at octave
            Semi_Tones(Index) := Quality_Interval'Pos(Quality_Interval'Last) +1  - Quality_Interval'Pos(Intervals(Index));
         else
            Semi_Tones(Index) := Quality_Interval'Pos(Intervals(Index + 1)) - Quality_Interval'Pos(Intervals(Index));
         end if;
      end loop;

      return Semi_Tones;
   end Convert_Intervals;

   function Convert_Intervals(Intervals : Interval_Array) return Present_Qualities_Array is
      Present_Qualities : Present_Qualities_Array := (Tone => True, others => False);
      Current_Distance : Notes_Interval := 0;
   begin
      for Index in Intervals'First..(Intervals'Last-1) loop
         Current_Distance := Current_Distance + Intervals(Index);
         Present_Qualities(Quality_Of(Current_Distance)) := True;
      end loop;

      return Present_Qualities;
   end Convert_Intervals;

   function Convert_Intervals(Intervals : Quality_Interval_Array) return Present_Qualities_Array is
      Present_Qualities : Present_Qualities_Array := (others => False);
   begin
      for Index in Intervals'Range loop
         Present_Qualities(Intervals(Index)) := True;
      end loop;

      return Present_Qualities;
   end Convert_Intervals;

   function Number_Of_Notes(Qualities : Present_Qualities_Array) return Natural is
      Number : Natural := 0;
   begin
      for Index in Qualities'Range loop
         if (Qualities(Index) = True) then
            Number := Number + 1;
         end if;
      end loop;

      return Number;
   end Number_Of_Notes;

   ------------
   -- Create --
   ------------
   procedure init(Object : in out NamedStructure; Name : in String; Intervals : in Interval_Array)
   is
   begin
      Object.Name := To_Unbounded_String(Name);
      Object.Structure := Convert_Intervals(Intervals);
      Object.Number_Of_Notes := Number_Of_Notes(Object.Structure);
   end Init;

   ------------
   -- Create --
   ------------
   procedure init(Object : in out NamedStructure; Name : in String; Intervals : in Quality_Interval_Array)
   is
   begin
      Object.Name:= To_Unbounded_String(Name);
      Object.Structure := Convert_Intervals(Intervals);
      Object.Number_Of_Notes := Number_Of_Notes(Object.Structure);
   end Init;

   ---------------
   -- Get_Notes --
   ---------------
   function Get_Notes(Beginning_Note : Notes.Note;Intervals : Interval_Array) return Set_Of_Notes
   is
      Result : Set_Of_Notes(1..Intervals'Length);
      Index : Positive := 1;
      Current_Note_Index : Notes.Note_Index := Notes.Note'Pos(Beginning_Note);

      use type Notes.Note_index;--To make + visible
   begin
      Result(Index) := Beginning_Note;
      Index := Index + 1;
      for Interval_index in Intervals'First..Intervals'Last - 1 loop
         for Tone in 1..Intervals(Interval_index) loop
            Current_Note_Index := Current_Note_Index + 1;
         end loop;
         Result(Index) := Notes.Note'Val(Current_Note_Index);
         Index := Index + 1;
      end loop;

      return Result;
   end Get_Notes;

   function Get_Notes(Beginning_Note : in Notes.Note;Quality_Intervals : in Quality_Interval_Array) return Set_Of_Notes
   is
      Result : Set_Of_Notes(1..Quality_Intervals'Length);
   begin
      for Index in Quality_Intervals'Range loop
         Result(Index) := Get_Note(Quality_Intervals(Index), Beginning_Note);
      end loop;

      return Result;
   end Get_Notes;

   function Get_Notes(The_structure : NamedStructure; Beginning_Note : in Notes.Note) return Set_Of_Notes
   is
      Result : Set_Of_Notes(1..The_Structure.Number_Of_Notes);
      Current_Index : Positive := 1;
   begin
      for Quality in The_Structure.Structure'range loop
         if (The_Structure.Structure(Quality) = True) then
            Result(Current_Index) := Get_Note(Quality, Beginning_Note);
            Current_Index := Current_Index + 1;
         end if;
      end loop;

      return Result;
   end Get_Notes;

   function Get_Structure(Object : NamedStructure) return Quality_Interval_Array is
      Returned : Quality_Interval_Array(1..Object.Number_Of_Notes);
      Index : Positive := 1;
   begin
      for Quality in Object.Structure'Range loop
         if Object.Structure(Quality) = True then
            Returned(Index) := Quality;
            Index := Index + 1;
         end if;
      end loop;
      return returned;
   end Get_Structure;

   function Get_Name(Object : NamedStructure) return String is
   begin
      return To_String(Object.Name);
   end Get_Name;

   function Contains(Container : NamedStructure; Object : Namedstructure) return Boolean is
   begin
      for Index in Present_Qualities_Array'Range loop
         if (Container.Structure(Index) = False and Object.Structure(Index) = True) then
            return False;
         end if;
      end loop;

      return True;
   end Contains;

   -- Function to test if a structure is equal to another one without taking name into account
   function Equals(Container : NamedStructure; Object : Namedstructure) return Boolean is
   begin
      if (Container.Structure = Object.Structure) then
         return True;
      else
         return False;
      end if;

   end Equals;

end Scalada.NamedStructures;
