--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Scalada.Notes; use Scalada.Notes;
with Scalada.Intervals; use Scalada.Intervals;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package Scalada.NamedStructures is

   -- The type representing notes of a scale or a chord
   type Set_Of_Notes is array(Positive range <>) of Note;

   -- Array of intervals defining a scale or a chord
   type Interval_Array is array(Positive range <>) of Notes_Interval;

   -- Array of Quality interval defining a scale or a chord
   type Quality_Interval_Array is array(Positive range <>) of Quality_Interval;

   type NamedStructure is abstract tagged private;

   -- Conversion between types
   function Convert_Intervals(Intervals : Quality_Interval_Array) return Interval_Array;

   -- Create a named structure object with a given name according to the list of intervals or quality intervals
   function Create(Name : in String; Intervals : in Interval_Array) return NamedStructure is abstract;
   function Create(Name : in String; Intervals : in Quality_Interval_Array) return NamedStructure is abstract;

   procedure Init(Object : in out NamedStructure; Name : in String; Intervals : in Interval_Array);
   procedure Init(Object : in out NamedStructure; Name : in String; Intervals : in Quality_Interval_Array);

   -- return the notes thanks to the tonic and the list of intervals as semi-tones
   function Get_Notes(Beginning_Note : in Note;Intervals : in Interval_Array) return Set_Of_Notes;

   -- return the notes  thanks to the tonic and the list of intervals as qualities
   function Get_Notes(Beginning_Note : in Note; Quality_intervals : in Quality_Interval_Array) return Set_Of_Notes;

   function Get_Notes(The_structure : NamedStructure; Beginning_Note : in Note) return Set_Of_Notes;

--   function Get_Structure(Object : NamedStructure'Class) return Interval_Array;

   -- Accessor to the quality structure as quality_interval_array
   function Get_Structure(Object : NamedStructure) return Quality_Interval_Array;

   -- Accessor to the name of the structure
   function Get_Name(Object : NamedStructure) return String;

   -- Function to test if a structure is contained in another one
   function Contains(Container : NamedStructure; Object : Namedstructure) return Boolean;

   -- Function to test if a structure is equal to another one without taking name into account
   function Equals(Container : NamedStructure; Object : Namedstructure) return Boolean;

private
   -- Array of present qualities defining a scale
   type Present_Qualities_Array is array(Quality_Interval) of Boolean;

   type NamedStructure is abstract tagged record
      Name : Unbounded_String;
      Structure : Present_Qualities_Array;
      Number_Of_Notes : Natural;
   end record;

end Scalada.NamedStructures;
