--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package body Scalada.Scales is

   -----------------------
   -- Utility functions --
   -----------------------
   procedure Check_Intervals_Validity(Intervals : in Interval_Array) is
      use Notes;
      Sum : Natural := 0;
      Number_Of_Tones : constant := Note'Pos(Note'Last) - Note'Pos(Note'First) + 1;
   begin
      for Index in Intervals'Range loop
         Sum := Sum + Natural(Intervals(Index));
      end loop;

      if Sum /= Number_Of_Tones then
         raise Wrong_Interval_Values with "Sum of intervals should be equal to " & Natural'Image(Number_Of_Tones);
      end if;

   end Check_Intervals_Validity;

   ------------------------
   -- Declared functions --
   ------------------------
   overriding
   function Create(Name : in String; Intervals : in Interval_Array) return Scale is
      Scale_Object : Scale;
   begin
      Check_Intervals_Validity(Intervals);
      NamedStructures.Init(NamedStructures.NamedStructure(Scale_Object), Name, Intervals);
      return Scale_Object;
   end Create;

   overriding
   function Create(Name : in String; Intervals : in Quality_Interval_Array) return Scale is
      Scale_Object : Scale;
   begin
      NamedStructures.Init(NamedStructure(Scale_Object), Name, Intervals);
      return Scale_Object;
   end Create;

   function Get_Notes(Beginning_Note : in Notes.Note;Intervals : in Interval_Array) return Set_Of_Notes
   is
   begin
      --Test for validity before anything else
      Check_Intervals_Validity(Intervals);
      return NamedStructures.Get_Notes(Beginning_Note, Intervals);
   end Get_Notes;

   function Get_Notes(Beginning_Note : in Notes.Note; Quality_Intervals : in Quality_Interval_Array) return Set_Of_Notes
   is
   begin
      --Test for validity before anything else
      Check_Intervals_Validity(Convert_Intervals(Quality_Intervals));
      return NamedStructures.Get_Notes(Beginning_Note, Quality_Intervals);
   end Get_Notes;

   overriding
   function Get_Notes(The_Scale : in Scale; Beginning_Note : in Notes.Note) return Set_Of_Notes
   is
   begin
      return NamedStructures.Get_Notes(NamedStructure(The_Scale),Beginning_Note);
   end Get_Notes;

end Scalada.Scales;
