--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.
with Scalada.Notes; use Scalada.Notes;
with Scalada.Intervals; use Scalada.Intervals;
with Scalada.NamedStructures; use Scalada.NamedStructures;

package Scalada.Scales is

   -- The scale type
   type Scale is new NamedStructure with null record;

   type Scale_Access is access Scale;

   -- Exception raised when the sum of intervals provided is not equal to the number of
   -- notes present in the chromatic scale. In this case, scale would be incomplete
   -- (i.e. would not be octave wide)
   Wrong_Interval_Values : exception;

   -- Create a scale object with a given name according to the list of intervals or quality intervals
   overriding
   function Create(Name : in String; Intervals : in Interval_Array) return Scale;
   
   overriding
   function Create(Name : in String; Intervals : in Quality_Interval_Array) return Scale;

   -- return the scale notes thanks to the tonic and the list of intervals as semi-tones
   function Get_Notes(Beginning_Note : in Note;Intervals : in Interval_Array) return Set_Of_Notes;

   -- return the scale thanks to the tonic and the list of intervals as qualities
   function Get_Notes(Beginning_Note : in Note; Quality_intervals : in Quality_Interval_Array) return Set_Of_Notes;

   -- return the notes for a scale according to the tonic
   overriding function Get_Notes(The_Scale : Scale; Beginning_Note : in Note) return Set_Of_Notes;

end Scalada.Scales;
