--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Ada.Streams.Stream_Io; use Ada.Streams.Stream_Io;
with Ada.Containers.Indefinite_Hashed_Maps; use Ada.Containers;

package body Scalada.Scales_Repository.Stream_IO is

   Repo_Filename : constant String := "scales_repo.db";

   use Scales_Map;

   ----------
   -- Save --
   ----------

   procedure Save is
      File_Desc : File_Type;

      procedure Save_Scale(Position : Cursor) is
      begin
         Scale'Write(Stream(File_Desc), Scales_Map.Element(Position));
      end Save_Scale;

   begin
      Create(File_Desc, Name => Repo_Filename);
      Scales_Map.Iterate(Repository,Save_Scale'Access);
      Close(File_Desc);
   end Save;


   ----------
   -- Load --
   ----------

   procedure Load is
      File_Desc : File_Type;
      Local_Scale : Scale;
   begin
      Clear;
      Open(File_Desc, Mode => In_File, Name => Repo_Filename);
      while not End_Of_File(File_Desc) loop
         Scale'Read(Stream(File_Desc), Local_Scale);
         Register(Local_Scale);
      end loop;
      Close(File_Desc);
   end Load;

end Scalada.Scales_Repository.Stream_IO;
