--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Ada.Containers.Indefinite_Hashed_Maps; use Ada.Containers;
with Ada.Strings.Bounded; use Ada.Strings.Bounded;
with Ada.Strings.Hash;

package body Scalada.Scales_Repository is

   use Scales_Map;

   --------------
   -- Register --
   --------------

   procedure Register (One_Scale : Scale) is
   begin
      -- Push the scale in the repo
      Insert(Repository,Get_Name(One_Scale), One_Scale);
   exception
      when Constraint_Error =>
         raise Already_Existing_Scale;
   end Register;

   ---------------
   -- Get_Scale --
   ---------------

   function Get_Scale (Name_Of_Scale : String) return Scale is
   begin
      return Element(Repository, Name_Of_Scale);
   exception
      when Constraint_Error =>
         raise Unknown_Scale;
   end Get_Scale;

   ----------------------
   -- Number_Of_Scales --
   ----------------------
   function Number_Of_Scales return Natural is
   begin
      return Natural(Length(Repository));
   end Number_Of_Scales;

   ---------------------------
   -- Get_Registered_Scales --
   ---------------------------
   function Get_Registered_Scales return Scales_Array is
      Get_Registered_Scales : Scales_Array(1..Number_Of_Scales);
      Index : Scales_Map.Cursor := Scales_Map.First(Repository);
      Scales_Index : Natural := 1;
   begin
      while Scales_Map.Has_Element(Index) loop
         Get_Registered_Scales(Scales_Index) := Scales_Map.Element(Index);
         Next(Index);
         Scales_Index := Scales_Index + 1;
      end loop;

      return Get_Registered_Scales;
   end Get_Registered_Scales;

   -----------
   -- Clear --
   -----------
   procedure Clear is
   begin
      Scales_Map.Clear(Repository);
   end Clear;

end Scalada.Scales_Repository;
