--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Scalada.Scales; use Scalada.Scales;
with Scalada.NamedStructures; use Scalada.NamedStructures;
with Ada.Containers.Indefinite_Hashed_Maps; use Ada.Containers;
with Ada.Strings.Bounded; use Ada.Strings.Bounded;
with Ada.Strings.Hash;

package Scalada.Scales_Repository is

   Already_Existing_Scale : exception;
   Unknown_Scale : exception;

   type Scales_Array is array(Natural range <>) of Scale;

   -- Procedure to add a scale to the repository
   -- can throw the Already_Existing_Scale exception
   -- if a scale with the same name
   procedure Register(One_Scale : Scale);

   -- Function to return a scale
   -- Can throw the Unknown_Scale exception
   function Get_Scale(Name_Of_Scale : String) return Scale;

   -- Function that returns the number of scales present in the repository
   function Number_Of_Scales return Natural;

   -- Function to retrieve all scales stored
   function Get_Registered_Scales return Scales_Array;

   -- Procedure to clean the content of the repository
   procedure Clear;

private
   package Scales_Map is new Ada.Containers.Indefinite_Hashed_Maps
     (Key_Type => String,
      Element_Type => Scale,
      Hash => Ada.Strings.Hash,
      Equivalent_Keys => "=" );

      Repository : Scales_Map.Map;

end Scalada.Scales_Repository;
