--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Gtk.Enums;
with Gtk.Paned; use Gtk.Paned;
with Gtk.Button; use Gtk.Button;
with Gtk.Handlers; use Gtk.Handlers;
with Gtk.Menu; use Gtk.Menu;
with Gtk.Menu_Bar; use Gtk.Menu_Bar;
with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk.Box;               use Gtk.Box;
with Gtk.Drawing_Area; use Gtk.Drawing_Area;
with Gtk.Aspect_Frame; use Gtk.Aspect_Frame;

with Ada.Text_Io; use Ada.Text_Io;
with Ada.Characters.Handling; use Ada.Characters.Handling;
with Ada.Exceptions;  use Ada.Exceptions;
with Ada.Numerics; use Ada.Numerics;

with Scalada.Scale_Finder_Panels; use Scalada.Scale_Finder_Panels;
with Scalada.Intervals_Panels; use Scalada.Intervals_Panels;
with Scalada.Fretboard_Panels; use Scalada.Fretboard_Panels;

with Scalada.Guitar_Fretboards; use Scalada.Guitar_Fretboards;
with Scalada.Scales; use Scalada.Scales;
with Scalada.Scale_Finder; use Scalada.Scale_Finder;
with Scalada.Notes.Text_Io; use Scalada.Notes.Text_Io;

package body Scalada.Application_Window is

   -- Declare callbacks handler
   package Scale_Finder_Cb is new Callback(Scale_Finder_Panel_Record);

   procedure Notes_Changed_Cb(Panel : access Scale_Finder_Panel_Record'Class) is
   begin
      Put_Line("Notes changed in app");
   end Notes_Changed_Cb;

   procedure Scale_Selected_Cb(Panel : access Scale_Finder_Panel_Record'Class) is
      Board : Fretboard := Create;
   begin
      Put_Line("Scale selected, sending to guitar fretboard");
      Activate_Notes(Board,Get_Notes(Selected_Scale(Panel.all).Matching_Scale,
                                     Selected_Scale(Panel.all).Tonic));

      for String_Index in Board'Range(1) loop
         for Fret_Index in Board'Range(2) loop
            if Board(String_Index,Fret_Index).Fretted then
               Put(To_Upper(Latin_Name(Board(String_Index,Fret_Index).Fret_Note)) & " ");
            else
               Put(To_Lower(Latin_Name(Board(String_Index,Fret_Index).Fret_Note)) & " ");
            end if;
         end loop;
         New_Line;
      end loop;

   exception
      when No_Scale_Selected => Put_Line("Ouups, nothing selected");
      when Ex: others => Put_Line (Exception_Information(Ex));

   end Scale_Selected_Cb;
   
   -------------------
   -- Create_Window --
   -------------------

   function Create_Window return Gtk_Window is
      Window : Gtk_Window;
      Main_Pane : Gtk_Paned;
      Aspect : Gtk_Aspect_Frame;
      
      Scale_Finder_Widget : Scale_Finder_Panel;

--      Intervals : Intervals_Panel;

      Fretboard_Ui : Fretboard_Panel;

      Bar : Gtk_Menu_Bar;
      File_Bar : Gtk_Menu_Item;
      Menu : Gtk_Menu;
      Quit_Item : Gtk_Menu_Item;
      Box1 : Gtk_Box;

   begin
      Gtk.Window.Gtk_New
        (Window   => Window,
         The_Type => Gtk.Enums.Window_Toplevel);

      --  From Gtk.Widget:
      Gtk.Window.Set_Title (Window => Window, Title  => "Scales application");

--      Gtk_New_Vbox (Box1, False, 0);
--      Add (window, Box1);
--      Pack_Start (Box1, Bar, False, False, 0);

      -- Create menus
--      Gtk_New(Menu);
--      Gtk_New(Bar);
--      Gtk_New(File_Bar, "File");
--      Gtk_New(Quit_Item, "Quit");

--      Append(Bar, File_Bar);

--      Append(Menu, Quit_Item);

--      Set_Submenu(File_Bar, Menu);

      -- Create the main panel
      Gtk_New_Vpaned(Main_Pane);

      -- Get the upper part
      Gtk_New(Fretboard_ui);
	    
      Set_USize(Fretboard_Ui,0, 60);

      --Add1(Main_Pane, Intervals);
      Add1(Main_Pane, Fretboard_ui);

      -- Get the lower part
      Gtk_New(Scale_Finder_widget);

      Add2(Main_Pane, Scale_Finder_widget);

      Add(Window, Main_Pane);

      -- connect handlers
      Scale_Finder_Cb.Connect(Scale_Finder_Widget,
                              Scale_Finder_Panels.Signal_Notes_Changed,
                              Scale_Finder_Cb.To_Marshaller(Notes_Changed_Cb'Access));

      Scale_Finder_Cb.Connect(Scale_Finder_Widget,
                              Scale_Finder_Panels.Signal_Scale_Selected,
                              Scale_Finder_Cb.To_Marshaller(Scale_Selected_Cb'Access));

      return Window;
   end Create_Window;

end Scalada.Application_Window;
