with Gtk.Handlers; use Gtk.Handlers;
with Glib.Object; use Glib.Object;
with Gtkada.Types;    use Gtkada.Types;
with Glib; use Glib;

with Gdk.Gc; use Gdk.Gc;
with Gdk.Window; use Gdk.Window;
with Gdk.Drawable; use Gdk.Drawable;

package body Scalada.Fretboard_Panels is

   --  This pointer will keep a pointer to the C 'class record' for
   --  gtk. To avoid allocating memory for each widget, this may be done
   --  only once, and reused
   Class_Record : GObject_Class := Uninitialized_Class;

   --  Array of the signals created for this widget
   Signals : Chars_Ptr_Array := Null_Array;

   -- Internal callback to handle the expose event
   package Event_Cb is new Return_Callback (Fretboard_Panel_Record, Boolean);

   -- Internal procedure to draw the widget
   procedure Draw_Board(Panel : access Fretboard_Panel_Record'Class) is separate;

   --Handle expose event
   function Expose_Cb(Panel : access Fretboard_Panel_Record'Class) return Boolean is
   begin
      Draw_Board(Panel);
      return False;
   end Expose_Cb;

   -------------
   -- Gtk_New --
   -------------
   procedure Gtk_New
     (Panel : out Fretboard_Panel;
      Number_Of_Strings : String_Number := DEFAULT_NUMBER_OF_STRINGS;
      Number_Of_Frets : Fret_Number := DEFAULT_NUMBER_OF_FRETS)
   is
   begin
      Panel := new Fretboard_Panel_Record;
      Panel.Number_Of_Strings := Number_Of_Strings;
      Panel.Number_Of_Frets := Number_Of_Frets;

      Fretboard_Panels.Initialize(Panel);
   end Gtk_New;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize
     (Panel : access Fretboard_Panel_Record'Class)
   is
   begin
      Gtk.Aspect_Frame.Initialize(Aspect_Frame => Panel,
                               Label => "",
                               Xalign => 0.0,
                               Yalign => 0.0,
                               Ratio => 8.0,
                               Obey_Child => False);

      Glib.Object.Initialize_Class_Record
        (Panel, Signals, Class_Record, "Fretboard_Panel");

      Gtk_New(Panel.Drawing);
      Add(Panel, Panel.Drawing);

      -- connect the expose_event to draw with Cairo
      Event_Cb.Connect (Panel, "expose_event", Event_Cb.To_Marshaller (Expose_Cb'Access));

   end Initialize;

   ---------------------
   -- Highlight_Notes --
   ---------------------

   procedure Highlight_Notes (Board: Fretboard) is
   begin
      --  Generated stub: replace with real body!
      raise Program_Error;
   end Highlight_Notes;

end Scalada.Fretboard_Panels;
