--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Gtk.Handlers; use Gtk.Handlers;
with Glib.Object; use Glib.Object;
with Gtkada.Types;    use Gtkada.Types;
with Scalada.Notes.Text_IO; use Scalada.Notes.Text_IO;

package body Scalada.Notes_Panels is

   --  This pointer will keep a pointer to the C 'class record' for
   --  gtk. To avoid allocating memory for each widget, this may be done
   --  only once, and reused
   Class_Record : GObject_Class := Uninitialized_Class;

   --  Array of the signals created for this widget
   Signals : Chars_Ptr_Array := Null_Array + "notes_changed";

   package Internal_Cb is new Callback (Notes_Panel_Record);
   --  The type of callbacks for the signals above. This is used only to
   --  emit the signals.

   package Check_Cb is new User_Callback (Gtk_Check_Button_Record, Notes_Panel);

   procedure Checked(button : access Gtk_Check_Button_Record'Class; Panel : Notes_Panel) is
   begin
      if (Get_Active(Button) = True) then
         Panel.Selected_Notes_Counter := Panel.Selected_Notes_Counter + 1;
      else
         Panel.Selected_Notes_Counter := Panel.Selected_Notes_Counter - 1;
      end if;

      Internal_Cb.Emit_By_Name (Panel, Signal_Notes_Changed);
   end Checked;

   procedure Gtk_New (Panel : out Notes_Panel) is
   begin
      Panel := new Notes_Panel_Record;
      Initialize(Panel, "Notes");
   end Gtk_New;

   procedure Initialize (Panel : access Notes_Panel_Record'Class; Label : UTF8_String :="") is
   begin
      Gtk.Frame.Initialize(Panel,Label);

      Glib.Object.Initialize_Class_Record
        (Panel, Signals, Class_Record, "Notes_Panel");

      Gtk.Table.Gtk_New(Panel.Table,6 , 2, Homogeneous => False);

      for Note_Index in Panel.Check_Array'Range loop
         Gtk_New(Panel.Check_Array(Note_Index),International_Name(Note_Index) & " (" & Latin_Name(Note_Index) & ")");
         Gtk.Table.Attach(Panel.Table, Panel.Check_Array(Note_Index),
                          0, 1,
                          Notes.Note'Pos(Note_Index),
                          (Notes.Note'Pos(Note_Index)) + 1);
         Check_Cb.Connect(Panel.Check_Array(Note_Index),  --  The object to connect to the handler
                          "clicked",               --  The name of the signal
                          Check_Cb.To_Marshaller(Checked'Access),  --  The signal handler
                          Notes_Panel(Panel)
                         );
      end loop;

      Add (Panel, Panel.Table);
      Show (Panel.Table);
   end Initialize;

   function Selected_Notes(Panel : in Notes_Panel_Record) return Set_Of_Notes is
      Returned_Notes : Set_Of_Notes(1..Panel.Selected_Notes_Counter);
      Index : Natural := 1;
   begin
      for Notes_Index in Panel.Check_Array'Range loop
         if (Get_Active(Panel.Check_Array(Notes_Index)) = True) then
            Returned_Notes(Index) := Notes_Index;
            Index := Index + 1;
         end if;
      end loop;

      return Returned_Notes;
   end Selected_Notes;

end Scalada.Notes_Panels;
