--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Glib.Object; use Glib.Object;
with Gtkada.Types; use Gtkada.Types;
with Gtk.Tree_Model; use Gtk.Tree_Model;
with Gtk.Tree_Selection; use Gtk.Tree_Selection;
with Gtk.Cell_Renderer_text; use Gtk.Cell_Renderer_text;
with Gtk.Tree_View_Column; use Gtk.Tree_View_Column;
with Glib.Values; use Glib.Values;
with Gtk.Paned; use Gtk.Paned;
with Gtk.Handlers; use Gtk.Handlers;
with Gtk.Toggle_Button;
with Gtk.Enums; use Gtk.Enums;
with Gtk.Tree_Selection;
with Gtk.Scrolled_Window; use Gtk.Scrolled_Window;

with Ada.Exceptions; use Ada.Exceptions;

with Scalada.Scales; use Scalada.Scales;
with Scalada.Notes.Text_IO;
with Scalada.Scales_Repository;

with Scalada.Notes;

package body Scalada.Scale_Finder_Panels is

   --  This pointer will keep a pointer to the C 'class record' for
   --  gtk. To avoid allocating memory for each widget, this may be done
   --  only once, and reused
   Class_Record : GObject_Class := Uninitialized_Class;

   --  Array of the signals created for this widget
   Signals : Chars_Ptr_Array := "scale_selected" + "notes_changed";

   -- Callback for sending event only
   --  The type of callbacks for the signals above. This is used only to
   --  emit the signals.
   package Internal_Cb is new Callback (Scale_Finder_Panel_Record);

   -- internal callbacks
   package Notes_Panel_Cb is new User_Callback(Notes_Panel_Record, Scale_Finder_Panel);

   package Match_Button_Cb is new User_Callback(Gtk_Check_Button_Record, Scale_Finder_Panel);

   package Tree_Cb is new User_Callback(Gtk_Tree_View_Record, Scale_Finder_Panel);

   -------------------------------
   ------- Generic function ------
   -------------------------------
   -- declare a generic update depending on scales finder matching function
   generic
      with function matches(Selected_Notes : in Set_Of_Notes)
                          return Matching_Scales_Array;
   procedure Update_Scales_List(Panel : access Notes_Panel_Record'Class; Parent_Panel : Scale_Finder_Panel);

   -- Procedure to update the model for list
   -- scales can be containing scales or matching scales depending on matches generic parameter
   procedure Update_Scales_List(Panel : access Notes_Panel_Record'Class; Parent_Panel : Scale_Finder_Panel) is
      Scales_Found : Matching_Scales_Array := matches(Selected_Notes(Panel.all));
      Index_gint : Gint := 0;
      Iter : Gtk_Tree_Iter;
   begin
      Clear(Parent_Panel.Store);
      for Index in Scales_Found'Range loop
         declare
            Scale_Name_Value : GValue;
            Tonic_Value : GValue;
            Tonic_Position : GValue;
         begin
            Init(Scale_Name_Value, GType_String);
            Init(Tonic_Value, GType_String);
            Init(Tonic_Position, GType_Uint);

            Set_String(Scale_Name_Value, Scales.Get_Name(Scales_Found(Index).Matching_Scale));
            Set_String(Tonic_Value, Notes.Text_IO.Latin_Name(Scales_Found(Index).Tonic));
            Set_Uint(Tonic_Position, Notes.Note'Pos(Scales_Found(Index).Tonic));

            Insert_With_Valuesv(Parent_Panel.Store,Iter, Index_gint, (0, 1, 2),(Scale_Name_Value, Tonic_Value, Tonic_Position));
            Index_gint := Index_gint + 1;
         end;
      end loop;

      Columns_Autosize(Parent_Panel.Tree);
   end Update_Scales_List;

   -- generic procedure instanciation for matching scales
   procedure Update_Matching_Scales_List is new Update_Scales_List(Matches => Scale_Finder.Matching_Scales);

   -- generic procedure instanciation for containing scales
   procedure Update_Containing_Scales_List is new Update_Scales_List(Matches => Containing_Scales);

   ------- End of generic function definition

   -- Procedure used when notes panel changed notes selected
   procedure Notes_Changed(Panel : access Notes_Panel_Record'Class; Parent_Panel : Scale_Finder_Panel) is

   begin
      if (Get_Active(Parent_Panel.Match_Button) = False) then
         Update_Containing_Scales_List(Panel, Parent_Panel);
      else
         Update_Matching_Scales_List(Panel, Parent_Panel);
      end if;

      -- Send an event to containing controller
      Internal_Cb.Emit_By_Name(Parent_Panel, Signal_Notes_Changed);

   end Notes_Changed;


   -- procedure when matching button changed
   procedure Matching_Button_Clicked(Button : access Gtk_Check_Button_Record'Class; Parent_Panel : Scale_Finder_Panel) is
   begin
      if (Get_Active(Button) = False) then
         Update_Containing_Scales_List(Parent_Panel.Notes_Panel_Widget, Parent_Panel);
      else
         Update_Matching_Scales_List(Parent_Panel.Notes_Panel_Widget, Parent_Panel);
      end if;
   end Matching_Button_Clicked;

   -- procedure to emit a signal when a row has been selected
   procedure Row_Clicked(tree : access Gtk_Tree_View_Record'Class; Parent_Panel : Scale_Finder_Panel) is
   begin
      Internal_Cb.Emit_By_Name(Parent_Panel, Signal_Scale_Selected);
   end Row_Clicked;

   -------------
   -- Gtk_New --
   -------------
   procedure Gtk_New (Panel : out Scale_Finder_Panel) is
   begin
      Panel := new Scale_Finder_Panel_Record;
      Initialize(Panel, "Scale Finder");
   end Gtk_New;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize
     (Panel : access Scale_Finder_Panel_Record'Class;
      Label : UTF8_String :="")
   is
      Notes_Paned : Gtk_Paned;
      Main_Pane : Gtk_Paned;

      Column_Scale : Gtk_Tree_View_Column;
      Column_tonic : Gtk_Tree_View_Column;
      Column_Tonic_Position : Gtk_Tree_View_Column;

      -- Viewport to show the treeview
      Viewport : Gtk_Scrolled_Window;

      Renderer : Gtk_Cell_Renderer_Text;
      Fake : Gint;
      Iter : Gtk_Tree_Iter;
   begin
      Gtk.Frame.Initialize(Panel,Label);

      Glib.Object.Initialize_Class_Record
        (Panel, Signals, Class_Record, "Scale_Finder_Panel");

      -- Create Main pane
      Gtk_New_HPaned(Main_Pane);

      -- Create notes panel with button
      Gtk_New_VPaned(Notes_Paned);

      -- Create the notes panel
      Gtk_New(Panel.Notes_Panel_Widget);

      -- Add it
      Add(Notes_Paned, Panel.Notes_Panel_widget);

      -- create the match button
      Gtk_New(Panel.Match_Button, "Matching");

      -- Add it
      Add(Notes_Paned, Panel.Match_Button);

      -- Add notes_paned to main
      Add(Main_Pane, Notes_Paned);

      -- create the tree panel
      Gtk_New(Panel.Store, (GType_String, GType_String, GType_Uint));
      Gtk_New(Panel.Tree, Panel.store);

      -- change the selection mode for this tree
      Gtk.Tree_Selection.Set_Mode(Get_Selection(Panel.Tree),Gtk.Enums.Selection_Single);

      Gtk_New(Column_Scale);
      Set_Title(Column_Scale, "Scale");

      Gtk_New(Renderer);
      Pack_Start(Column_Scale, Renderer, true);
      add_attribute(Column_scale, renderer, "text", 0);

      Fake := Append_Column(Panel.tree, Column_Scale);

      Gtk_New(Column_Tonic);
      Set_Title(Column_Tonic, "Tonic");

      Gtk_New(Renderer);
      Pack_Start(Column_Tonic, Renderer, true);
      add_attribute(Column_Tonic, renderer, "text", 1);

      Fake := Append_Column(Panel.tree, Column_Tonic);

      Append(Panel.Store,Iter);

      Gtk_New(Column_Tonic_Position);
      Set_Title(Column_Tonic_Position, "Tonic Position");

      Gtk_New(Renderer);
      Pack_Start(Column_Tonic_Position, Renderer, true);
      add_attribute(Column_Tonic_Position, renderer, "text", 2);
      -- Make this column invisible as it contains technical data only
      Set_Visible(Column_Tonic_Position, False);

      Fake := Append_Column(Panel.tree, Column_Tonic_Position);

      Append(Panel.Store,Iter);

      -- create the viewport and add the treeview to it
      Gtk_New(Viewport);
      Add(Viewport, Panel.Tree);

      -- Add it
      Add(Main_Pane, Viewport);
      Add(Panel, Main_Pane);

      Show(Panel.Notes_Panel_Widget);
      Show(Panel.Tree);
      Show(Panel.Match_Button);

      -- connect handlers
      Notes_Panel_Cb.Connect(Panel.Notes_Panel_Widget,
                             Notes_Panels.Signal_Notes_Changed,
                             Notes_Panel_Cb.To_Marshaller(Notes_Changed'Access),
                             Scale_Finder_Panel(Panel)
                            );

      Match_Button_Cb.Connect(Panel.Match_Button,
                              Gtk.Toggle_Button.Signal_Toggled,
                              Match_Button_Cb.To_Marshaller(Matching_Button_Clicked'Access),
                              Scale_Finder_Panel(Panel)
                             );

      Tree_Cb.Connect(Panel.Tree,
                      Gtk.Tree_view.Signal_Cursor_Changed,
                      Tree_Cb.To_Marshaller(Row_Clicked'Access),
                      Scale_Finder_Panel(Panel)
                     );
   end Initialize;

   --------------------
   -- Selected_Notes --
   --------------------

   function Selected_Notes
     (Panel : in Scale_Finder_Panel_Record)
      return Set_Of_Notes
   is
   begin
      return Selected_Notes(Panel.Notes_Panel_Widget.all);
   end Selected_Notes;

   --------------------
   -- Selected_Scale --
   --------------------

   function Selected_Scale
     (Panel : in Scale_Finder_Panel_Record)
      return Matching_Scale
   is
      -- get the selection model used
      Selection_Model : Gtk_Tree_Selection := Get_Selection(Panel.Tree);

      -- Data model
      Model : Gtk_Tree_Model;
      --iterator
      Iter : Gtk_Tree_Iter;

   begin
      Get_Selected(Selection_Model, Model, Iter);

      if Iter = Null_Iter then
         Raise_Exception (No_Scale_Selected'Identity, "no scale was selected");
      else
         declare
            Name_Of_Scale : GValue;
            Tonic : GValue;
            Tonic_Position : GValue;
         begin
            -- get the scale from the path
            Get_Value(Model, Iter, 0, Name_Of_Scale);
            Get_Value(Model, Iter, 1, Tonic);
            Get_Value(Model, Iter, 2, Tonic_Position);
            declare
                  -- returned matching scale
               Returned_Scale : Matching_Scale :=
                 Matching_Scale'(Scalada.Notes.Note'Val(Get_Uint(Tonic_Position)),
                                 Scalada.Scales_Repository.Get_Scale(Get_String(Name_Of_Scale)));
            begin
               return Returned_scale;
            end;
         end;
      end if;
   end Selected_Scale;

end Scalada.Scale_Finder_Panels;
