--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Ahven; use Ahven;

with Scalada.Intervals; use Scalada.Intervals;
with Scalada.Notes; use Scalada.Notes;
with Scalada.NamedStructures; use Scalada.NamedStructures;
with Scalada.Chords; use Scalada.Chords;

package body Chords_Tests is

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (T : in out Test) is
   begin
      Set_Name (T, "Chords tests");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Interval'Access, "Chords notes with Interval tests");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Quality'Access, "Chords notes with Quality tests");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Chord'Access, "Chord notes with Chord object");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Quality_Chord'Access, "Chord notes with Quality Chord object");
   end Initialize;

   ----------------------------------
   -- Test_Get_Notes_With_Interval --
   ----------------------------------

   procedure Test_Get_Notes_With_Interval is
   begin
  La_Majeur:
      declare
         Chords_Interval : Interval_Array := (4, 3, 4);
         Result_Chord : Set_Of_Notes := (A, C_Sharp, E);
        begin
           Assert(Get_Notes(A, Chords_Interval) = Result_Chord, "A major chord should be composed of A, C#, E");
        end La_Majeur;

   end Test_Get_Notes_With_Interval;

   ---------------------------------
   -- Test_Get_Notes_With_Quality --
   ---------------------------------

   procedure Test_Get_Notes_With_Quality is
   begin
  La_Majeur:
      declare
         Chord_Interval : Quality_Interval_Array := (Tone, Major_Third, Perfect_Fifth);
         Result_Chord : Set_Of_Notes := (A, C_Sharp, E);
        begin
           Assert(Get_Notes(A, Chord_Interval) = Result_Chord, "A major chord should be composed of A, C# and E");
        end La_Majeur;
   end Test_Get_Notes_With_Quality;

   procedure Test_Get_Notes_With_Chord
   is
      Chord_Interval : Interval_Array := (4, 3, 4);
      My_Chord : Chord := Create("Major Chord", Chord_Interval);
      Result_Chord : Set_Of_Notes := (A, C_Sharp, E);
   begin
--      Chords.Text_IO.Display(Get_Notes(A, My_chord));
      Assert(Get_Notes(My_Chord, A) = Result_Chord, "A major chord should be composed of A, C#, E");
   end Test_Get_Notes_With_Chord;

   procedure Test_Get_Notes_With_Quality_Chord
   is
   begin

  Major:
      declare
         Chord_Interval : Quality_Interval_Array := (Tone, Major_Third, Perfect_Fifth);
         My_Chord : Chord := Create("Major Chord", Chord_Interval);
         Result_Chord : Set_Of_Notes := (A, C_Sharp, E);
      begin
         Assert(Get_Notes(My_Chord, A) = Result_Chord, "A major chord should be composed of A, C#, E");
      end Major;

  Minor:
      declare
         Chord_Interval : Quality_Interval_Array := (Tone, Minor_Third, Perfect_Fifth);
         My_Chord : Chord := Create("Minor chord", Chord_Interval);
         Result_Chord : Set_Of_Notes := (A, C, E);
      begin
         Assert(Get_Notes(My_Chord, A) = Result_Chord, "A minor chord should be composed of A, C, E");
      end Minor;

   end Test_Get_Notes_With_Quality_Chord;

end Chords_Tests;
