--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Ahven; use Ahven;
with Scalada.Guitar_Fretboards; use Scalada.Guitar_Fretboards;
with Scalada.Notes; use Scalada.Notes;
with Scalada.NamedStructures; use Scalada.NamedStructures;

package body Guitar_Fretboard_Tests is

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (T : in out Test) is
   begin
      Set_Name (T, "Guitar fretboard tests");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Create_standard'Access, "Test standard guitar fretboard with standard tuning");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Create_Custom'Access, "Test 7 string, 22 frets, Drop-D tuning guitar fretboard");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Activation'Access, "Test activation on fretboard with standard guitar (only 12 frets)");

   end Initialize;

   procedure Test_Create_Standard is
      My_Fretboard : Fretboard := Create;
   begin
      -- Check several positions
      Assert(My_Fretboard(1,5).Fret_Note = A, "Fifth fret of the E string must be A");
      Assert(My_Fretboard(2,5).Fret_Note = D, "Fifth fret of the A string must be D");
      Assert(My_Fretboard(3, 12).Fret_Note = D, "Twelfth fret of the D string should be D");
      Assert(My_Fretboard(4, 10).Fret_Note = F, "Tenth fret of the G string should be F");
      Assert(My_Fretboard(5, 5).Fret_Note = E, "Tenth fret of the B string should be E");
      Assert(My_Fretboard(6, 5).Fret_Note = A, "Fifth fret of the higher E string should be A");
      Assert(My_Fretboard(4, 0).Fret_Note = G, "Opened on G string should be G");
   end Test_Create_Standard;

   procedure Test_Create_Custom is
      My_Fretboard : Fretboard := Create(7, 22, Tuning'(1 => B,
                                                        2 => D,
                                                        3 => A,
                                                        4 => D,
                                                        5 => G,
                                                        6 => B,
                                                        7 => E
                                                       ));
   begin
      -- Check several positions
      Assert(My_Fretboard(1,5).Fret_Note = E, "Fifth fret of the B string must be E");
      Assert(My_Fretboard(2,0).Fret_Note = D, "Open the D string must be D");
      Assert(My_Fretboard(2,5).Fret_Note = G, "Open the D string must be G");
   end Test_Create_Custom;

   procedure Test_Activation is
      My_Fretboard : Fretboard := Create(Frets_Number => 12);
      Notes_To_Highlight : Set_Of_Notes := (A, G);
   begin
      Activate_Notes(My_Fretboard, Notes_To_Highlight);

      Assert(My_Fretboard(1,3).Fretted = True, "Third fret on first string should be turned on");
      Assert(My_Fretboard(1,5).Fretted = True, "Fifth fret on first string should be turned on");
      Assert(My_Fretboard(2,0).Fretted = True, "No fret on second string should be turned on");
      Assert(My_Fretboard(4,0).Fretted = True, "No fret on fourth string should be turned on");
      Assert(My_Fretboard(3,0).Fretted = False, "No fret on third string should be turned off");
   end Test_Activation;

end Guitar_Fretboard_Tests;
