--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

-- my_tests.adb
with Ahven; use Ahven;

with Scalada.Intervals; use Scalada.Intervals;
with Scalada.Notes; use Scalada.Notes;

package body Intervals_Tests is
   procedure Initialize (T : in out Test) is
   begin
      Set_Name (T, "Intervals tests");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Interval_Value'Access, "Semi-tone Interval tests");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Quality_Of'Access, "Quality interval tests");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Note'Access, "Quality interval tests");
   end Initialize;

   procedure Test_Interval_Value is
   begin
      Assert (Interval_Value(Tone) = 0 , "Tone should be null interval");
      Assert (Interval_Value(Minor_Second) = 1, "Second should be a semi-tone interval");
      Assert (Interval_Value(Perfect_Fourth) = 5, "Perfect Fourth should be a 5 semi-tones");
   end Test_Interval_Value;

   procedure Test_Quality_Of is
   begin
      Assert(Quality_Of(4) = Major_Third , "4 semi-tones interval must be a Major third");
      Assert(Quality_Of(7) = Perfect_Fifth, "7 semi-tones interval must be a Perfect_Fifth");

      Assert(Quality_Of(F, C) = Perfect_Fourth, "F is the perfect fourth of C");
      Assert(Quality_Of(G, A) = Minor_Seventh, "G is the minor seventh of A");
   end Test_Quality_Of;

   procedure Test_Get_Note is
   begin
      Assert(Get_Note(Major_Third, A) = C_Sharp, "Major third of A should be C#");
   end Test_Get_Note;

end Intervals_Tests;
