--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Scalada.Scale_Finder; use Scalada.Scale_Finder;
with Scalada.NamedStructures; use Scalada.NamedStructures;
with Scalada.Notes; use Scalada.Notes;
with Scalada.Notes.Text_io; use Scalada.Notes.Text_io;
with Ahven; use Ahven;
with Scalada.Scales.Text_IO; use Scalada.Scales.Text_IO;
with Ada.Text_Io; use Ada.Text_Io;
with Scalada.Chords; use Scalada;
with Scalada.Intervals; use Scalada.Intervals;

package body Scale_Finder_Tests is

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (T : in out Test) is
   begin
      Set_Name (T, "Scale finder tests");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Find_Scale'Access, "Find a specific scale");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Find_Enclosing_Scale'Access, "Find an specific enclosing scale");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Find_Enclosing_Scale_For_Chords'Access, "Find an specific enclosing scale from chord objects");

   end Initialize;

   ---------------------
   -- Test_Find_Scale --
   ---------------------

   procedure Test_Find_Scale is
      My_Access : Matching_Scales_Array := Matching_Scales(Set_Of_Notes'(C, D, A, G, E));
   begin
      if My_Access'Length = 0 then
         Fail("Should detect at least one scale");
      end if;

  Major :
      declare
         Matching : Matching_Scales_Array := Matching_Scales(Set_Of_Notes'(G, D, A, F, C, E, B));
      begin
         if Matching'Length = 0 then
            Fail("Should detect at least one scale");
         end if;

         Put_Line("Matching scales : ");
         for index in Matching'Range loop
            Display(Matching(Index).Matching_Scale);
         end loop;

      end Major;

   end Test_Find_Scale;

   -------------------------------
   -- Test_Find_Enclosing_Scale --
   -------------------------------
   procedure Test_Find_Enclosing_Scale is
      Matching : Matching_Scales_Array := Containing_Scales(Set_Of_Notes'(G, D, A, F, C, E));
   begin
      if Matching'Length = 0 then
         Fail("Should detect at least one scale");
      end if;

      Put_Line("Containing scales : ");
      for index in Matching'Range loop
         Put_Line(Scales.Get_Name(Matching(Index).Matching_Scale) & Latin_Name(Matching(Index).Tonic));
      end loop;
   end Test_Find_Enclosing_Scale;

   -----------------------------------------
   -- Test_Find_Enclosing_Scale_For_Chord --
   -----------------------------------------
   procedure Test_Find_Enclosing_Scale_For_Chords is
      Matching : Matching_Scales_Array := Containing_Scales(Chords_Array'(
                                                                          Specific_Chord'(A, Chords.Create("Major chord", Quality_Interval_Array'(TONE, Major_Third, Perfect_Fifth))),
                                                                          Specific_Chord'(D, Chords.Create("Major chord", Quality_Interval_Array'(TONE, Major_Third, Perfect_Fifth)))
                                                                         )
                                                           );
   begin

      if Matching'Length = 0 then
         Fail("Should detect at least one scale");
      end if;

      Put_Line("Containing scales : ");
      for index in Matching'Range loop
         Put_Line(Scales.Get_Name(Matching(Index).Matching_Scale) & Latin_Name(Matching(Index).Tonic));
      end loop;
   end Test_Find_Enclosing_Scale_For_Chords;

end Scale_Finder_Tests;
