--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Scalada.Scales; use Scalada.Scales;
with Scalada.Intervals; use Scalada.Intervals;
with Scalada.Scales_Repository; use Scalada.Scales_Repository;
with Scalada.NamedStructures; use Scalada.NamedStructures;
with Scalada.Scales.Text_Io; use Scalada.Scales.Text_Io;
with Scalada.Notes; use Scalada;
with Ada.Text_Io; use Ada.Text_Io;
with Scalada.Scales_Repository.Stream_IO; use Scalada.Scales_Repository.Stream_IO;

package body Scales_Repository_Tests is

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (T : in out Test) is
   begin
      Set_Name (T, "Scales repository tests");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Register_Scale'Access, "Register a scale");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Scale'Access, "Test for retrieving a Scale");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Registered_Scales'Access, "Test for getting the whole content");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Unknown_Scale'Access, "Test for getting a scale that does not exist");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Register_Already_Existing'Access, "Test for getting a scale that already exists");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Save_Load'Access, "Test for saving/loading the repository");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Clear'Access, "Test for cleaning the repository");

   end Initialize;

   --------------------
   -- Test_Get_Scale --
   --------------------

   procedure Test_Get_Scale is
      Scale_To_Retrieve : Scale;
   begin
      Scale_To_Retrieve := Get_Scale("Major scale");
   exception
      when Unknown_Scale =>
         Ahven.Fail("Unknown_Scale exception should not be raised");
   end Test_Get_Scale;

   -------------------
   -- Test_Register --
   -------------------

   procedure Test_Register_Scale is
   begin
      Register(Create("Minor Pentatonic scale", Quality_Interval_Array'(Tone, Minor_Third, Perfect_Fourth, Perfect_Fifth, Minor_Seventh)));
      Register(Create("Minor Blues scale", Quality_Interval_Array'(Tone, Minor_Third, Perfect_Fourth, Tritone, Perfect_Fifth, Minor_Seventh)));
      Register(Create("Major scale", Quality_Interval_Array'(Tone, Second, Major_Third, Perfect_Fourth , Perfect_Fifth, Major_Sixth, Major_Seventh)));
      Register(Create("Minor scale", Interval_Array'(2, 1, 2, 2, 1, 2, 2)));
      Register(Create("Minor Harmonic scale", Interval_Array'(2, 1, 2, 2, 1, 3, 1)));
      Register(Create("Ionian scale", Interval_Array'(2, 2, 1, 2, 2, 2, 1)));
      Register(Create("Dorian scale", Interval_Array'(2, 1, 2, 2, 2, 1, 2)));
      Register(Create("Phrygian scale", Interval_Array'(1, 2, 2, 2, 1, 2, 2)));
      Register(Create("Lydian scale", Interval_Array'(2, 2, 2, 1, 2, 2, 1)));
      Register(Create("Mixolydian scale", Interval_Array'(2, 2, 1, 2, 2, 1, 2)));
      Register(Create("Aeolian scale", Interval_Array'(2, 1, 2, 2, 1, 2, 2)));
      Register(Create("Locrian scale", Interval_Array'(1, 2, 2, 1, 2, 2, 2)));
      Register(Create("Ascending melodic minor scale", Interval_Array'(2, 1, 2, 2, 2, 2, 1)));
      Register(Create("Descending melodic minor scale", Quality_Interval_Array'(Tone, Second , Minor_Third, Perfect_Fourth, Perfect_Fifth, Minor_Sixth, Minor_Seventh)));
   exception
      when Already_Existing_Scale =>
         Ahven.Fail("Already_Existing_Scale exception should not be raised");
   end Test_Register_Scale;

   --------------------------------
   -- Test_Get_Registered_Scales --
   --------------------------------

   procedure Test_Get_Registered_Scales is
      Registered_Scales : Scales_Array := Get_Registered_Scales;
   begin
      --  for Index in Registered_Scales'Range loop
      --     display(Registered_Scales(Index));
      --  end loop;
      Ahven.Assert(Registered_Scales'Length = Number_Of_scales, "There must be the same number of scales in the repository as returned by Get_registered_Scales");
   end Test_Get_Registered_Scales;

   ----------------------------
   -- test_Get_Unknown_Scale --
   ----------------------------
   procedure Test_Get_Unknown_Scale is
      Scale_To_Retrieve : Scale;
   begin
      Scale_To_Retrieve := Get_Scale("Whatever scale");
      Ahven.Fail("Unknown_Scale exception should have been raised");
   exception
      when Unknown_Scale =>
         null;
   end Test_Get_Unknown_Scale;

   ------------------------------------
   -- Test_Register_Already_Existing --
   ------------------------------------
   procedure Test_Register_Already_Existing is
   begin
      Register(Create("Minor Pentatonic scale", Quality_Interval_Array'(Tone, Minor_Third, Perfect_Fourth, Perfect_Fifth, Minor_Seventh)));
      Ahven.Fail("Already_Existing_Scale exception should have been thrown");
   exception
      when Already_Existing_Scale =>
         null;
   end Test_Register_Already_Existing;

   procedure Test_Save_Load is
   begin
      Save;
      Load;
      Ahven.Assert(Number_Of_Scales > 0, "There should be at least one scale in the repository");

      --Display the content
      --  declare
      --     Scales_Loaded : Scales_Array := Get_Registered_Scales;
      --  begin
      --     Put_Line("scales loaded : ");
      --     for Index in Scales_Loaded'Range loop
      --        Display(Scales_Loaded(Index));
      --        Put_Line("for A : ");
      --        Display(Get_Notes(Scales_Loaded(Index), Notes.A));
      --     end loop;
      --  end;
   end Test_Save_Load;

   procedure Test_Clear is
   begin
      Clear;
      Ahven.Assert(Number_Of_Scales = 0, "Scales repository should be empty");
      Load;
   end Test_Clear;

end Scales_Repository_Tests;
