--  This software is distributed under the following license know as
--  The modified BSD license
--
--  Redistribution and use in source and binary forms, with or without
--  modification, are permitted provided that the following conditions are
--  met:

--      (1) Redistributions of source code must retain the above copyright
--      notice, this list of conditions and the following disclaimer. 

--      (2) Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in
--      the documentation and/or other materials provided with the
--      distribution.  
    
--      (3)The name of the author may not be used to
--      endorse or promote products derived from this software without
--      specific prior written permission.

--  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
--  DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
--  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
--  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
--  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
--  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
--  POSSIBILITY OF SUCH DAMAGE.

with Ahven; use Ahven;

with Scalada.Intervals; use Scalada.Intervals;
with Scalada.Notes; use Scalada.Notes;
with Scalada.NamedStructures; use Scalada.NamedStructures;
with Scalada.Scales; use Scalada.Scales;
use Scalada;

package body Scales_Tests is

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize (T : in out Test) is
   begin
      Set_Name (T, "Scales tests");

      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Interval'Access, "Scale notes with Interval tests");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Quality'Access, "Scale notes with Quality tests");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Wrong_Intervals'Access, "Scale notes with wrong intervals");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Scale'Access, "Scales notes with Scale object");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Quality_Scale'Access, "Scales notes with Quality Scale object");
      Ahven.Framework.Add_Test_Routine
        (T, Test_Get_Notes_With_Mixed_Quality_Scale'Access, "Scales notes with mixed quality Scale object");
   end Initialize;

   ----------------------------------
   -- Test_Get_Notes_With_Interval --
   ----------------------------------

   procedure Test_Get_Notes_With_Interval is
   begin
  Gamme_Majeure:
      declare
         Scale_Interval : Interval_Array := (2, 2, 1, 2, 2, 2, 1);
         Result_Scale : Set_Of_Notes := (C, D, E, F, G, A, B);
        begin
           Assert(Scales.Get_Notes(C, Scale_Interval) = Result_Scale, "C major scale should be composed of C, D, E, F, G, A, B");
        end Gamme_Majeure;

   end Test_Get_Notes_With_Interval;

   ---------------------------------
   -- Test_Get_Notes_With_Quality --
   ---------------------------------

   procedure Test_Get_Notes_With_Quality is
   begin
  Gamme_Majeure:
      declare
         Scale_Interval : Quality_Interval_Array := (Tone, Second, Major_Third, Perfect_Fourth , Perfect_Fifth, Major_Sixth, Major_Seventh);
         Result_Scale : Set_Of_Notes := (C, D, E, F, G, A, B);
        begin
           Assert(Scales.Get_Notes(C, Scale_Interval) = Result_Scale, "C major scale should be composed of C, D, E, F, G, A, B");
        end Gamme_Majeure;
   end Test_Get_Notes_With_Quality;

   -----------------------------------------
   -- Test_Get_Notes_With_Wrong_Intervals --
   -----------------------------------------
   procedure Test_Get_Notes_With_Wrong_Intervals is
   begin
  Gamme_Majeure:
      declare
         Scale_Interval : Interval_Array := (2, 2, 1, 2, 2, 2, 2);
         Result_Scale : Set_Of_Notes(Scale_Interval'Range);
        begin
           Result_Scale := Scales.Get_Notes(C, Scale_Interval);
           Fail("Should have thrown an exception here");

        exception
           when Ex : Wrong_Interval_Values =>
              null;--Nothing to do, test passed
        end Gamme_Majeure;

   end Test_Get_Notes_With_Wrong_Intervals;

   procedure Test_Get_Notes_With_Scale
   is
      Scale_Interval : Interval_Array := (2, 2, 1, 2, 2, 2, 1);
      My_Scale : Scale := Create("Major scale", Scale_Interval);
      Result_Scale : Set_Of_Notes := (C, D, E, F, G, A, B);
   begin
      Assert(Get_Notes(My_Scale, C) = Result_Scale, "C major scale should be composed of C, D, E, F, G, A, B");
   end Test_Get_Notes_With_Scale;

   procedure Test_Get_Notes_With_Quality_Scale
   is
   begin

  Major:
      declare
         Scale_Interval : Quality_Interval_Array := (Tone, Second, Major_Third, Perfect_Fourth , Perfect_Fifth, Major_Sixth, Major_Seventh);
         My_Scale : Scale := Create("Major scale", Scale_Interval);
         Result_Scale : Set_Of_Notes := (C, D, E, F, G, A, B);
      begin
         Assert(Get_Notes(My_Scale, C) = Result_Scale, "C major scale should be composed of C, D, E, F, G, A, B");
      end Major;

  Minor:
      declare
         Scale_Interval : Quality_Interval_Array := (Tone, Second, Minor_Third, Perfect_Fourth , Perfect_Fifth, Minor_Sixth, Major_Seventh);
         My_Scale : Scale := Create("Harmonic Minor scale", Scale_Interval);
         Result_Scale : Set_Of_Notes := (C, D, D_Sharp, F, G, G_Sharp, B);
      begin
         Assert(Get_Notes(My_Scale, C) = Result_Scale, "C harmonic minor scale should be composed of C, D, D#, F, G, G#, B");
      end Minor;

   end Test_Get_Notes_With_Quality_Scale;

   procedure Test_Get_Notes_With_Mixed_Quality_Scale is
      Scale_Interval : Quality_Interval_Array := (Tone, Minor_Third, Perfect_Fourth, Second, Minor_Sixth, Perfect_Fifth, Major_Seventh);
      My_Scale : Scale := Create("Harmonic Minor scale", Scale_Interval);
      Result_Scale : Set_Of_Notes := (C, D, D_Sharp, F, G, G_Sharp, B);
   begin
      Assert(Get_Notes(My_Scale, C) = Result_Scale, "C harmonic minor scale should be composed of C, D, D#, F, G, G#, B");
   end Test_Get_Notes_With_Mixed_Quality_Scale;

end Scales_Tests;
